/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
#ifndef CAMERA_H
#define CAMERA_H

#include <GL/glew.h>
#include <glm/glm.hpp>
#include "trackball.h"
class CPhysicalCamera
{
public:
    GLfloat lastquat[4];
    GLfloat curquatY[4];
    GLfloat curquatX[4];
    glm::vec3 vEye, vView, vUp;
    glm::vec3 vMove;
    double fSpeed;
    glm::vec3 vSpeed;
    double fSetTime;
    double fTotalMoveTime;
    bool bRoate_camera_flag;
    bool bMove_camera_flag;
    bool bRotateCameraModel;
    bool bMoveCmeraModel;
    bool bAerialView;
    bool bAontrolCamera;
    double fSensitivity;
    glm::mat4 mRotateY;
    glm::mat4 mRotateX;
    glm::mat4 mLook;

    double fRotationAngleCube , fRotationAnglePyramid;
    double fCubeRotationSpeed , fPyramidRotationSpeed;

    double AerialViewY;
    double fRotateCameraXSpeed , fRotateCameraYSpeed;
    double PIover180;
    double flength;
    glm::vec3 vModelCenterPos;
    void setLook(glm::mat4 m);
    glm::mat4& getLook();
    void setPerspective(glm::mat4 Perspective);
    const glm::mat4 getPerspective();
    void rotateCamera(double horizontal, double vertical, double planar,
                      double time);
    void moveCamera(double x, double y, double z, double time);
    void SetRotationAngle(double fRotateCube = 0.0f,
                          double fRotatePyramid = 0.0f,
                          double fCubeRotateSpeed = 0.0f,
                          double fPyramidRotateSpeed = 0.0f);
    float GetAngleX();
    glm::mat4 getModelViewMatrix(glm::mat4 lookat,
                                 glm::mat4& matrix,
                                 float fMoveTime,
                                 float fPreTime);
    void RotateObjectMouse(double horizontal, double vertical,
                           double planar);
    void RotateCamera(double horizontal, double vertical,
                      double planar);
    void removeRotateObject();
    void buildRotMatrix(glm::mat4& m, float q[4]);

    CPhysicalCamera();
    CPhysicalCamera(glm::vec3 a_vEye, glm::vec3 a_vView, glm::vec3 a_vUp,
                    double a_fSpeed = 25.0f, double a_fSensitivity = 0.1f);
private:
    int iForw, iBack, iLeft, iRight;
    glm::mat4 viewMatrix;
    glm::mat4 mProjection;
};

#endif
